'''
General utilities for the MPESA functions
'''

from .exceptions import *
from django_daraja.models import AccessToken
import requests
import json
from django.utils import timezone
from decouple import config, UndefinedValueError
import os
from requests import Response

class MpesaResponse(Response):
	response_description = ''
	error_code = None
	error_message = ''

def mpesa_response(r):
	'''
	Creates MpesaResponse object from requests.Response object
	
	Arguments:
		r {requests.Response} -- The response to convert
	'''

	r.__class__ = MpesaResponse
	json_response = r.json()
	r.response_description = json_response.get('ResponseDescription', '')
	r.error_code = json_response.get('errorCode')
	r.error_message = json_response.get('errorMessage', '')
	return r

def mpesa_config(key):
	'''
	Gets Mpesa configuration variable with the matching key, otherwise returns MpesaConfigurationException
	
	Arguments:
		key {str} -- The configuration key
	'''

	try:
		value = config(key)
	except UndefinedValueError:
		# Check key in settings file
		raise MpesaConfigurationException('Mpesa environment not configured properly - ' + key + ' not found')

	return value


def api_base_url():
	'''
	Gets the base URL depending on the development environment (sandbox or production)
	'''

	mpesa_environment = mpesa_config('MPESA_ENVIRONMENT')

	if mpesa_environment == 'sandbox':
		return 'https://sandbox.safaricom.co.ke/'
	elif mpesa_environment == 'production':
		return 'https://api.safaricom.co.ke/'
	else:
		raise MpesaConfigurationException('Mpesa environment not configured properly - MPESA_ENVIRONMENT should be sandbox or production')

def generate_access_token_request(consumer_key = None, consumer_secret = None):
	'''
	Makes call to OAuth API
	'''

	url = api_base_url() + 'oauth/v1/generate?grant_type=client_credentials'
	consumer_key = consumer_key if consumer_key is not None else mpesa_config('MPESA_CONSUMER_KEY') 
	consumer_secret = consumer_secret if consumer_secret is not None else mpesa_config('MPESA_CONSUMER_SECRET')

	try:
		r = requests.get(url, auth=(consumer_key, consumer_secret))
	except requests.exceptions.ConnectionError:
		raise MpesaConnectionError('Connection failed')
	except Exception:
		return ex.message
	
	return r

def generate_access_token():
	'''
	Parses OAuth response to generate access token, then updates database access token value
	'''

	r = generate_access_token_request()
	token = json.loads(r.text)['access_token']

	AccessToken.objects.all().delete()
	access_token = AccessToken.objects.create(token=token)

	return access_token

def mpesa_access_token():
	'''
	Generates access token if the current one has expired or if token is non-existent
	Otherwise returns existing access token
	'''

	# access_token = generate_access_token()

	access_token = AccessToken.objects.first()
	if access_token == None:
		# No access token found
		access_token = generate_access_token()
	else:
		delta = timezone.now() - access_token.created_at
		minutes = (delta.total_seconds()//60)%60
		print('minutes: ', minutes)
		if minutes > 50:
			# Access token expired
			access_token = generate_access_token()	
	
	return access_token.token

def format_phone_number(phone_number):
	'''
	Formats phone number into the format 2547XXXXXXXX
	'''

	if len(phone_number) < 9:
		raise IllegalPhoneNumberException('Phone number too short')
	else:
		return '254' + phone_number[-9:]



# -*- coding: utf-8 -*-
'''
Test STK Push
'''

from django.test import TestCase
from django_daraja.mpesa.core import MpesaClient
from decouple import config

class MpesaStkPushTestCase(TestCase):

	cl = MpesaClient()

	def test_stk_push_success(self):
		'''
		Test successful STK push
		'''

		phone_number = config('LNM_PHONE_NUMBER')
		amount = 1
		account_reference = 'reference'
		transaction_desc = 'Description'
		callback_url = 'https://darajambili.herokuapp.com/express-payment'
		response = self.cl.stk_push(phone_number, amount, account_reference, transaction_desc, callback_url)
		self.assertEqual(response.response_description, 'Success. Request accepted for processing')

	def test_stk_push_invalid_phone_number(self):
		'''
		Test STK push with invalid phone number
		'''

		phone_number = '254733000'
		amount = 1
		account_reference = 'reference'
		transaction_desc = 'Description'
		callback_url = 'https://darajambili.herokuapp.com/express-payment'
		response = self.cl.stk_push(phone_number, amount, account_reference, transaction_desc, callback_url)
		self.assertEqual(response.status_code, 500)
		



'''
Exceptions raised during usage of the MPESa API
'''

class IllegalPhoneNumberException(Exception):
	'''
	Raised when phone number is in illegal format.
	'''
	pass


class MpesaConnectionError(Exception):
	'''
	Raised when connection has an error
	'''
	pass

class MpesaConfigurationException(Exception):
	'''
	Raised when Mpesa environment variables are not configured properly
	'''
	pass


import re
import os

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="django-daraja",
    version="0.0.1",
    description="A python django library for interacting with the MPESA Daraja API.",
    long_description=
"""
This is a django module based on the MPESA daraja API. Use it to interact with the MPESA API in a simplified manner, abstracting the details for a simple, intuitive usage experience, as well as extensive error handling.    

Read the documentation at https://django-admin.readthedocs.io

Daraja API documentation can be found at https://developer.safaricom.co.ke""",
    author="Martin Mogusu",
    author_email="martinmogusu@gmail.com",
    url="https://github.com/martinmogusu/django-daraja",
    download_url="https://github.com/martinmogusu/django-daraja.git",
    license="MIT License",
    packages=[
        "django_daraja",
    ],
    include_package_data=True,
    install_requires=[
        "Django>=1.11",
        "python-decouple",
        "requests"
    ],
    tests_require=[
        "nose",
        "coverage",
    ],
    zip_safe=False,
    test_suite="tests.runtests.start",
    classifiers=[
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ]
)